@@ -6,6 +6,7 @@ from .base_asdu import (
 from .app_asdu import *
 from .app_asdu import INSTANT_VALUES_OBJECTS
 import math
+from datetime import datetime
 
 from six import with_metaclass
 
@@ -87,10 +88,22 @@ class ASDUDirectionUnknown(Exception):
     pass
 
 
+class NoContentTimeoutException(Exception):
+
+    def __str__(self):
+        return 'Timeout: To many Time Without Content'
+
+
 class AppLayer(with_metaclass(ABCMeta)):
 
+    def __init__(self):
+        self.last_content_heartbeat = self.connection_start = datetime.now()
+        self.set_content_timeout(300)  # in seconds
+
+
     def initialize(self, link_layer):
         self.link_layer = link_layer
+        self.last_content_heartbeat = datetime.now()
 
     def send_user_data(self, asdu):
         pass
@@ -98,6 +111,25 @@ class AppLayer(with_metaclass(ABCMeta)):
     def get_user_data(self):
         pass
 
+    def set_content_timeout(self, timeout):
+        self.max_content_timeout = timeout
+
+    def reset_content_received(self):
+        logger.info("RESET Last Content Heartbeat, content received")
+        self.last_content_heartbeat = datetime.now()
+
+    def check_content_timeout(self):
+        now = datetime.now()
+        diff = (now - self.last_content_heartbeat).total_seconds()
+        logger.info("Last Content Heartbeat: {} seconds".format(diff))
+        logger.info("TOTAL Current Duration: {} seconds".format(self.get_connection_duration()))
+        if diff > self.max_content_timeout:
+            raise NoContentTimeoutException
+
+    def get_connection_duration(self):
+        now = datetime.now()
+        return (now - self.connection_start).total_seconds()
+
     def process_request(self, request_asdu):
         self.link_layer.send_frame(request_asdu)
         asdu_ack = self.link_layer.get_frame()
@@ -126,6 +158,7 @@ class AppLayer(with_metaclass(ABCMeta)):
             yield asdu_resp
 
             if not isinstance(asdu_resp, VariableAsdu):
+                self.check_content_timeout()
                 continue
             if asdu_resp.causa_tm == 0x05 and asdu_resp.tipo in [
                 M_TA_VC_2.type, M_TA_VM_2.type, M_IT_TK_2.type, M_IT_TG_2.type,
@@ -155,6 +188,8 @@ class AppLayer(with_metaclass(ABCMeta)):
             else:
                 raise Exception('ERROR: Transmission cause unknown: {}'.format(asdu_resp.causa_tm))
 
+            self.check_content_timeout()
+
     def authenticate(self, clave_pm):
         #183
         asdu = self.create_asdu_request(C_AC_NA_2(clave_pm))
@@ -182,6 +217,7 @@ class AppLayer(with_metaclass(ABCMeta)):
         resps = list(self.process_request(asdu))
         for resp in self.process_requestresponse():
             if resp.tipo == M_IT_TG_2.type:
+                self.reset_content_received()
                 yield resp
 
     def read_incremental_values(self, start_date, end_date, register='profiles'):
@@ -197,6 +233,7 @@ class AppLayer(with_metaclass(ABCMeta)):
         resps = list(self.process_request(asdu))
         for resp in self.process_requestresponse():
             if isinstance(resp, VariableAsdu) and resp.tipo == M_IT_TK_2.type:
+                self.reset_content_received()
                 yield resp
 
     def read_datetime(self):
@@ -226,6 +263,7 @@ class AppLayer(with_metaclass(ABCMeta)):
         resps = list(self.process_request(asdu))
         for resp in self.process_requestresponse():
             if resp.tipo == M_TA_VC_2.type:
+                self.reset_content_received()
                 yield resp
 
     def stored_tariff_info(self, start_date, end_date, register=1):
@@ -239,6 +277,7 @@ class AppLayer(with_metaclass(ABCMeta)):
         resps = list(self.process_request(asdu))
         for resp in self.process_requestresponse():
             if resp.tipo == M_TA_VM_2.type:
+                self.reset_content_received()
                 yield resp
 
     def get_configuration(self):
